import os
import pystray
import threading
import time
from PIL import Image, ImageTk
import customtkinter as tk
from shutdown_on_lan.CtkScrollableTextDisabled import CtkScrollableTextDisabled

def Init() -> None:
    """ Create the system tray icon """

    tray = pystray.Icon("shutdown-on-lan")

    tray.icon = Image.open("python.png")
    tray.menu = pystray.Menu(
        pystray.MenuItem("Show Logs", ShowLogs),
        pystray.MenuItem("Quit", OnQuit)
    )

    tray.run()

def OnQuit(tray) -> None:
    """ Button to close the program from the system tray icon """

    tray.stop()
    os._exit(0)

def ShowLogs() -> None:
    """ Show the logs in a window """

    logViewerRoot = tk.CTk()
    
    logViewerRoot.geometry("600x400")
    logViewerRoot.title("shutdown-on-lan: Log Viewer")

    textArea = CtkScrollableTextDisabled(logViewerRoot, width=100, height=30)
    textArea.pack(expand=True, fill='both')

    logThread = threading.Thread(target=TailLogs, daemon=True, args=(textArea,))
    logThread.start()

    logViewerRoot.protocol("WM_DELETE_WINDOW", lambda: OnClosing(logViewerRoot, logThread))
    logViewerRoot.mainloop()

def TailLogs(textArea: CtkScrollableTextDisabled) -> None:
    """ Continuously read the log file and update the text in the window """

    global stopTail
    stopTail = False

    with open("server.log", "r") as logFile:
        while True:
            if stopTail:
                break

            logLine = logFile.readline()

            if logLine:
                textArea.insert(logLine)
            else:
                time.sleep(0.1)


def OnClosing(logViewerRoot: tk.CTk, logThread: threading.Thread) -> None:
    """ Close the log viewer window """

    global stopTail
    stopTail = True

    logThread.join()
    
    logViewerRoot.destroy()


from ast import arg
from fastapi import FastAPI
import threading

from shutdown_on_lan import Arguments
from shutdown_on_lan import Tray
from shutdown_on_lan.Shutdown import Shutdown
from shutdown_on_lan.Server import Server

app = FastAPI()

args = Arguments.init()

@app.get("/")
async def Home():
    """ Home route, mostly for testing """

    if args.hide_message:
        return {"message": "Disabled shutdown message"}
    else:
        return {"message": args.message}

@app.get("/shutdown")
async def GetShutdown():
    """ Shutdown route """

    Shutdown(args.message, args.hide_message, args.time)
    return {"message": args.message}

def main() -> None:
    """ Initialize the system tray icon and start the server """
    
    server_thread = threading.Thread(target=Server, args=(app, args))
    server_thread.start()

    with open("server.log", "w") as logFile:
        logFile.write("")

    Tray.Init()

if __name__ == "__main__":
    main()


from customtkinter import CTkScrollableFrame, CTkLabel
import tkinter as tk

class CtkScrollableTextDisabled():
    def __init__(self, root, width, height):
        self.root = root
        self.width = width
        self.height = height

        self.frame = CTkScrollableFrame(self.root)

        self.textArea = CTkLabel(self.frame, text="", width=self.width, height=self.height)

    def pack(self, expand, fill):
        self.frame.pack(expand=expand, fill=fill)
        self.textArea.pack(expand=expand, fill=fill)

    def insert(self, text):
        self.textArea.configure(state=tk.NORMAL)
        self.textArea.configure(text=self.textArea.cget("text") + text)
        self.textArea.configure(state=tk.DISABLED)

        self.frame._parent_canvas.yview_moveto(1.0)

    def destroy(self):
        self.frame.destroy()